/// <reference types="@cloudflare/workers-types" />

import type { APIActionRowComponent } from 'discord-api-types/v9';
import type { APIApplicationCommandAutocompleteInteraction } from 'discord-api-types/payloads/v9/_interactions/autocomplete';
import type { APIApplicationCommandOptionChoice } from 'discord-api-types/v9';
import type { APIBaseInteraction } from 'discord-api-types/v9';
import type { APIButtonComponent } from 'discord-api-types/v9';
import type { APIChatInputApplicationCommandInteraction } from 'discord-api-types/v9';
import type { APIEmbed } from 'discord-api-types/v9';
import type { APIEmbedField } from 'discord-api-types/v9';
import type { APIInteractionDataResolvedChannel } from 'discord-api-types/v9';
import type { APIInteractionResponseCallbackData } from 'discord-api-types/v9';
import type { APIMessage } from 'discord-api-types/v9';
import type { APIMessageApplicationCommandInteraction } from 'discord-api-types/v9';
import type { APIMessageButtonInteractionData } from 'discord-api-types/v9';
import type { APIMessageComponent } from 'discord-api-types/v9';
import type { APIMessageComponentEmoji } from 'discord-api-types/v9';
import type { APIMessageComponentInteractionData } from 'discord-api-types/v9';
import type { APIMessageSelectMenuInteractionData } from 'discord-api-types/v9';
import type { APIRole } from 'discord-api-types/v9';
import type { APISelectMenuComponent } from 'discord-api-types/v9';
import type { APISelectMenuOption } from 'discord-api-types/v9';
import type { APIUser } from 'discord-api-types/v9';
import type { APIUserApplicationCommandInteraction } from 'discord-api-types/v9';
import type { ChannelType as ChannelType_2 } from 'discord-api-types/v9';
import type { RESTGetAPIApplicationCommandPermissionsResult } from 'discord-api-types/v9';
import type { RESTGetAPIApplicationCommandResult } from 'discord-api-types/v9';
import type { RESTGetAPIApplicationCommandsResult } from 'discord-api-types/v9';
import type { RESTGetAPIApplicationGuildCommandResult } from 'discord-api-types/v9';
import type { RESTGetAPIApplicationGuildCommandsResult } from 'discord-api-types/v9';
import type { RESTGetAPIGuildApplicationCommandsPermissionsResult } from 'discord-api-types/v9';
import type { RESTGetAPIInteractionFollowupResult } from 'discord-api-types/v9';
import type { RESTGetAPIInteractionOriginalResponseResult } from 'discord-api-types/v9';
import type { RESTPatchAPIApplicationCommandJSONBody } from 'discord-api-types/v9';
import type { RESTPatchAPIApplicationCommandResult } from 'discord-api-types/v9';
import type { RESTPatchAPIApplicationGuildCommandJSONBody } from 'discord-api-types/v9';
import type { RESTPatchAPIApplicationGuildCommandResult } from 'discord-api-types/v9';
import type { RESTPatchAPIInteractionFollowupResult } from 'discord-api-types/v9';
import type { RESTPatchAPIInteractionOriginalResponseJSONBody } from 'discord-api-types/v9';
import type { RESTPatchAPIInteractionOriginalResponseResult } from 'discord-api-types/v9';
import type { RESTPostAPIApplicationCommandsJSONBody } from 'discord-api-types/v9';
import type { RESTPostAPIApplicationCommandsResult } from 'discord-api-types/v9';
import type { RESTPostAPIApplicationGuildCommandsJSONBody } from 'discord-api-types/v9';
import type { RESTPostAPIApplicationGuildCommandsResult } from 'discord-api-types/v9';
import type { RESTPostAPIInteractionFollowupJSONBody } from 'discord-api-types/v9';
import type { RESTPostAPIInteractionFollowupResult } from 'discord-api-types/v9';
import type { RESTPutAPIApplicationCommandPermissionsJSONBody } from 'discord-api-types/v9';
import type { RESTPutAPIApplicationCommandPermissionsResult } from 'discord-api-types/v9';
import type { RESTPutAPIApplicationCommandsJSONBody } from 'discord-api-types/v9';
import type { RESTPutAPIApplicationCommandsResult } from 'discord-api-types/v9';
import type { RESTPutAPIApplicationGuildCommandsJSONBody } from 'discord-api-types/v9';
import type { RESTPutAPIApplicationGuildCommandsResult } from 'discord-api-types/v9';
import type { Snowflake } from 'discord-api-types/v9';

export declare const $actionRow: unique symbol;

export declare const $actionRowChild: unique symbol;

export declare const $embed: unique symbol;

export declare const $field: unique symbol;

export declare const $update: unique symbol;

export declare interface AllowedMentionsProps {
    parse?: ValueOf<typeof AllowedMentionsTypes>[];
    roles?: Snowflake[];
    users?: Snowflake[];
    repliedUser?: boolean;
}

/** @see https://discord.com/developers/docs/resources/channel#allowed-mentions-object-allowed-mention-types */
export declare const AllowedMentionsTypes: {
    readonly ROLES: "roles";
    readonly USERS: "users";
    readonly EVERYONE: "everyone";
};

export declare type AnyCommand<Env = unknown> = () => AnyCommandHandler<Env>;

export declare type AnyCommandHandler<Env = unknown> = CommandHandler<Env> | UserCommandHandler<Env> | MessageCommandHandler<Env>;

export declare type APIAuth = APIBasicAuth | APIBearerAuth;

export declare interface APIBasicAuth {
    username: string;
    password: string;
}

export declare interface APIBearerAuth {
    bearer: string;
    expires?: number;
}

export declare class APIError extends Error {
    readonly method: string;
    readonly path: string;
    readonly code: number;
    constructor(method: string, path: string, code: number, message: string);
}

export declare type APIMessageComponentInteraction<Data> = APIBaseInteraction<typeof InteractionType.MESSAGE_COMPONENT, Data> & Required<Pick<APIBaseInteraction<typeof InteractionType.MESSAGE_COMPONENT, Data>, "channel_id" | "data" | "message">>;

/** @see https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-option-type */
export declare const ApplicationCommandOptionType: {
    readonly SUB_COMMAND: 1;
    readonly SUB_COMMAND_GROUP: 2;
    readonly STRING: 3;
    readonly INTEGER: 4;
    readonly BOOLEAN: 5;
    readonly USER: 6;
    readonly CHANNEL: 7;
    readonly ROLE: 8;
    readonly MENTIONABLE: 9;
    readonly NUMBER: 10;
};

/** @see https://discord.com/developers/docs/interactions/application-commands#application-command-object-application-command-types */
export declare const ApplicationCommandType: {
    readonly CHAT_INPUT: 1;
    readonly USER: 2;
    readonly MESSAGE: 3;
};

export declare function authorizeResponse(applicationId: string): Response;

export declare type AutocompleteHandler<T, Env> = (interaction: APIApplicationCommandAutocompleteInteraction, env: Env, ctx: ExecutionContext) => Awaitable<Choice<T>[]>;

export declare type Awaitable<T> = Promise<T> | T;

export declare type AwaitableGenerator<T = unknown, TReturn = any, TNext = unknown> = AsyncGenerator<T, TReturn, TNext> | Generator<T, TReturn, TNext>;

/** @see https://discord.com/developers/docs/interactions/application-commands#batch-edit-application-command-permissions */
export declare function bulkEditApplicationCommandPermissions(applicationId: Snowflake, guildId: Snowflake, permissions: RESTPutAPIApplicationCommandPermissionsJSONBody, auth: APIBearerAuth): Promise<RESTPutAPIApplicationCommandPermissionsResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#bulk-overwrite-global-application-commands */
export declare function bulkOverwriteGlobalApplicationCommands(applicationId: Snowflake, commands: RESTPutAPIApplicationCommandsJSONBody, auth: APIBearerAuth): Promise<RESTPutAPIApplicationCommandsResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#bulk-overwrite-guild-application-commands */
export declare function bulkOverwriteGuildApplicationCommands(applicationId: Snowflake, guildId: Snowflake, commands: RESTPutAPIApplicationGuildCommandsJSONBody, auth: APIBearerAuth): Promise<RESTPutAPIApplicationGuildCommandsResult>;

export declare function Button(props: ButtonProps): APIButtonComponent & {
    [$actionRowChild]: true;
};

export declare type ButtonProps = ButtonPropsWithID | ButtonPropsWithURL;

export declare interface ButtonPropsBase {
    emoji?: string | APIMessageComponentEmoji;
    disabled?: boolean;
    primary?: boolean;
    success?: boolean;
    danger?: boolean;
    children?: Child[];
}

export declare interface ButtonPropsWithID extends ButtonPropsBase {
    id: string;
}

export declare interface ButtonPropsWithURL extends ButtonPropsBase {
    url: string;
}

/** @see https://discord.com/developers/docs/interactions/message-components#button-object-button-styles */
export declare const ButtonStyle: {
    readonly PRIMARY: 1;
    readonly SECONDARY: 2;
    readonly SUCCESS: 3;
    readonly DANGER: 4;
    readonly LINK: 5;
};

export declare function call<Body, Result>(method: string, path: string, body?: Body, auth?: APIAuth): Promise<Result>;

export declare interface ChannelOption {
    types?: ChannelType_2[];
}

/** @see https://discord.com/developers/docs/resources/channel#channel-object-channel-types */
export declare const ChannelType: {
    readonly GUILD_TEXT: 0;
    readonly DM: 1;
    readonly GUILD_VOICE: 2;
    readonly GROUP_DM: 3;
    readonly GUILD_CATEGORY: 4;
    readonly GUILD_NEWS: 5;
    readonly GUILD_STORE: 6;
    readonly GUILD_NEWS_THREAD: 10;
    readonly GUILD_PUBLIC_THREAD: 11;
    readonly GUILD_PRIVATE_THREAD: 12;
    readonly GUILD_STAGE_VOICE: 13;
};

export declare type ChatInputCommands<Env = unknown> = Record<string, Command<Env> | Record<string, Command<Env> | Record<string, Command<Env>>>>;

export declare type Child = string | number | boolean | null | undefined;

export declare type Choice<T> = T | {
    name?: string;
    value: T;
};

export declare type ChoiceValue<Choice> = Choice extends {
    value: unknown;
} ? Choice["value"] : Choice;

export declare type Command<Env = unknown> = () => CommandHandler<Env>;

/** @see {CommandResponse} */
export declare type CommandHandler<Env = unknown> = (interaction: APIChatInputApplicationCommandInteraction, env: Env, ctx: ExecutionContext) => CommandResponse;

/**
 * - For `CHANNEL_MESSAGE_WITH_SOURCE`, return `MessageResponse`
 * - For `DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE`, yield nothing, then return `MessageResponse`
 * @see https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-callback-type
 */
export declare type CommandResponse = Awaitable<MessageResponse> | AwaitableGenerator<void, MessageResponse, never>;

/** @see {ComponentHandlerResponse}*/
export declare type ComponentHandler<Env = unknown, Data = APIMessageComponentInteractionData> = (interaction: APIMessageComponentInteraction<Data>, env: Env, ctx: ExecutionContext) => ComponentHandlerResponse;

/**
 * - For `CHANNEL_MESSAGE_WITH_SOURCE`, return `MessageResponse`
 * - For `DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE`, yield nothing, then return `MessageResponse`
 * - For `DEFERRED_UPDATE_MESSAGE`, yield `$update`, then return `MessageResponse`
 * - For `UPDATE_MESSAGE`, return `MessageResponse & { [$update]: true }`
 * @see https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-callback-type
 */
export declare type ComponentHandlerResponse = Awaitable<MessageResponse & {
    [$update]?: boolean;
}> | AwaitableGenerator<void | symbol, MessageResponse>;

/** @see https://discord.com/developers/docs/interactions/message-components#component-object-component-types */
export declare const ComponentType: {
    readonly ACTION_ROW: 1;
    readonly BUTTON: 2;
    readonly SELECT_MENU: 3;
};

export declare function createElement<Attributes, Children extends any[], Return>(component: (props: Attributes & {
    children: Children;
}) => Return, attributes: Attributes, ...children: Children): Return;

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#create-followup-message */
export declare function createFollowupMessage(applicationId: Snowflake, interactionToken: string, message: WithFileAttachments<RESTPostAPIInteractionFollowupJSONBody>): Promise<RESTPostAPIInteractionFollowupResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#create-global-application-command */
export declare function createGlobalApplicationCommand(applicationId: Snowflake, command: RESTPostAPIApplicationCommandsJSONBody, auth: APIBearerAuth): Promise<RESTPostAPIApplicationCommandsResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#create-guild-application-command */
export declare function createGuildApplicationCommand(applicationId: Snowflake, guildId: Snowflake, command: RESTPostAPIApplicationGuildCommandsJSONBody, auth: APIBearerAuth): Promise<RESTPostAPIApplicationGuildCommandsResult>;

export declare function createHandler<Env>(opts: Options<Env>): (request: Request, env: Env, ctx: ExecutionContext) => Promise<Response>;

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#delete-followup-message */
export declare function deleteFollowupMessage(applicationId: Snowflake, interactionToken: string, messageId: Snowflake): Promise<void>;

/** @see https://discord.com/developers/docs/interactions/application-commands#delete-global-application-command */
export declare function deleteGlobalApplicationCommand(applicationId: Snowflake, commandId: Snowflake, auth: APIBearerAuth): Promise<void>;

/** @see https://discord.com/developers/docs/interactions/application-commands#delete-guild-application-command */
export declare function deleteGuildApplicationCommand(applicationId: Snowflake, guildId: Snowflake, commandId: Snowflake, auth: APIBearerAuth): Promise<void>;

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#delete-original-interaction-response */
export declare function deleteOriginalInteractionResponse(applicationId: Snowflake, interactionToken: string): Promise<void>;

export declare function deployCommands<Env>(opts: Options<Env>): Promise<void>;

/** @see https://discord.com/developers/docs/interactions/application-commands#edit-application-command-permissions */
export declare function editApplicationCommandPermissions(applicationId: Snowflake, guildId: Snowflake, commandId: Snowflake, permissions: RESTPutAPIApplicationCommandPermissionsJSONBody, auth: APIBearerAuth): Promise<RESTPutAPIApplicationCommandPermissionsResult>;

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#edit-followup-message */
export declare function editFollowupMessage(applicationId: Snowflake, interactionToken: string, messageId: Snowflake, message: WithFileAttachments<RESTPatchAPIInteractionFollowupResult>): Promise<RESTPatchAPIInteractionFollowupResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#edit-global-application-command */
export declare function editGlobalApplicationCommand(applicationId: Snowflake, commandId: Snowflake, command: RESTPatchAPIApplicationCommandJSONBody, auth: APIBearerAuth): Promise<RESTPatchAPIApplicationCommandResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#edit-guild-application-command */
export declare function editGuildApplicationCommand(applicationId: Snowflake, guildId: Snowflake, commandId: Snowflake, command: RESTPatchAPIApplicationGuildCommandJSONBody, auth: APIBearerAuth): Promise<RESTPatchAPIApplicationGuildCommandResult>;

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#edit-original-interaction-response */
export declare function editOriginalInteractionResponse(applicationId: Snowflake, interactionToken: string, message: WithFileAttachments<RESTPatchAPIInteractionOriginalResponseJSONBody>): Promise<RESTPatchAPIInteractionOriginalResponseResult>;

export declare function Embed(props: EmbedProps): APIEmbed & {
    [$embed]: true;
};

export declare interface EmbedAuthorProps {
    name: string;
    url?: string;
    iconUrl?: string;
    proxyIconUrl?: string;
}

export declare interface EmbedFooterProps {
    text: string;
    iconUrl?: string;
    proxyIconUrl?: string;
}

export declare interface EmbedMediaProps {
    url?: string;
    proxyUrl?: string;
    width?: number;
    height?: number;
}

export declare interface EmbedProps {
    title?: string;
    url?: string;
    timestamp?: string | Date;
    color?: number;
    image?: string | EmbedMediaProps;
    thumbnail?: string | EmbedMediaProps;
    video?: string | EmbedMediaProps;
    footer?: string | EmbedFooterProps;
    provider?: string | EmbedProviderProps;
    author?: string | EmbedAuthorProps;
    children?: (Child | (APIEmbedField & {
        [$field]: true;
    }))[];
}

export declare interface EmbedProviderProps {
    name?: string;
    url?: string;
}

export declare function Field(props: FieldProps): APIEmbedField & {
    [$field]: true;
};

export declare interface FieldProps {
    name: string;
    inline?: boolean;
    children?: Child[];
}

export declare function Fragment<Children extends any[]>({ children, }: {
    children: Children;
}): Children;

/** @see https://discord.com/developers/docs/interactions/application-commands#get-application-command-permissions */
export declare function getApplicationCommandPermissions(applicationId: Snowflake, guildId: Snowflake, commandId: Snowflake, auth: APIBearerAuth): Promise<RESTGetAPIApplicationCommandPermissionsResult>;

/** @see https://discord.com/developers/docs/topics/oauth2#client-credentials-grant */
export declare function getBearerAuth(applicationId: Snowflake, applicationSecret: string): Promise<APIBearerAuth>;

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#get-followup-message */
export declare function getFollowupMessage(applicationId: Snowflake, interactionToken: string, messageId: Snowflake): Promise<RESTGetAPIInteractionFollowupResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#get-global-application-command */
export declare function getGlobalApplicationCommand(applicationId: Snowflake, commandId: Snowflake, auth: APIBearerAuth): Promise<RESTGetAPIApplicationCommandResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#get-global-application-commands */
export declare function getGlobalApplicationCommands(applicationId: Snowflake, auth: APIBearerAuth): Promise<RESTGetAPIApplicationCommandsResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#get-guild-application-command */
export declare function getGuildApplicationCommand(applicationId: Snowflake, guildId: Snowflake, commandId: Snowflake, auth: APIBearerAuth): Promise<RESTGetAPIApplicationGuildCommandResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#get-guild-application-command-permissions */
export declare function getGuildApplicationCommandPermissions(applicationId: Snowflake, guildId: Snowflake, auth: APIBearerAuth): Promise<RESTGetAPIGuildApplicationCommandsPermissionsResult>;

/** @see https://discord.com/developers/docs/interactions/application-commands#get-guild-application-commands */
export declare function getGuildApplicationCommands(applicationId: Snowflake, guildId: Snowflake, auth: APIBearerAuth): Promise<RESTGetAPIApplicationGuildCommandsResult>;

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#get-original-interaction-response */
export declare function getOriginalInteractionResponse(applicationId: Snowflake, interactionToken: string): Promise<RESTGetAPIInteractionOriginalResponseResult>;

export declare function handleLanding<Env>(opts: Options<Env>, request: Request): Promise<Response | void>;

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-response-object-interaction-callback-type */
export declare const InteractionResponseType: {
    readonly PONG: 1;
    readonly CHANNEL_MESSAGE_WITH_SOURCE: 4;
    readonly DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE: 5;
    readonly DEFERRED_MESSAGE_UPDATE: 6;
    readonly UPDATE_MESSAGE: 7;
    readonly APPLICATION_COMMAND_AUTOCOMPLETE_RESULT: 8;
};

/** @see https://discord.com/developers/docs/interactions/receiving-and-responding#interaction-object-interaction-type */
export declare const InteractionType: {
    readonly PING: 1;
    readonly APPLICATION_COMMAND: 2;
    readonly MESSAGE_COMPONENT: 3;
    readonly APPLICATION_COMMAND_AUTOCOMPLETE: 4;
};

export declare function Message(props: MessageProps): WithFileAttachments<APIInteractionResponseCallbackData> & {
    [$update]?: boolean;
};

export declare type MessageCommand<Env = unknown> = () => MessageCommandHandler<Env>;

/** @see {CommandResponse} */
export declare type MessageCommandHandler<Env = unknown> = (interaction: APIMessageApplicationCommandInteraction, env: Env, ctx: ExecutionContext, message: APIMessage) => CommandResponse;

export declare type MessageCommands<Env = unknown> = Record<string, MessageCommand<Env>>;

export declare interface MessageProps {
    update?: boolean;
    attachments?: File[];
    tts?: boolean;
    ephemeral?: boolean;
    allowedMentions?: AllowedMentionsProps;
    children?: (Child | (APIEmbed & {
        [$embed]: true;
    }) | (APIActionRowComponent & {
        [$actionRow]: true;
    }) | (APIButtonComponent & {
        [$actionRowChild]: true;
    }) | (APISelectMenuComponent & {
        [$actionRowChild]: true;
    }))[];
}

export declare type MessageResponse = WithFileAttachments<APIInteractionResponseCallbackData>;

export declare function normaliseChoices<T extends string | number>(choices?: ReadonlyArray<Choice<T>>): APIApplicationCommandOptionChoice[] | undefined;

export declare interface NumericOption {
    min?: number;
    max?: number;
}

export declare function Option(props: OptionProps): APISelectMenuOption;

export declare interface OptionalChoicesOption<Choices extends ReadonlyArray<Choice<string | number>>> {
    required?: false;
    choices: Choices;
}

export declare interface OptionalOption<T, Env> {
    required?: false;
    autocomplete?: AutocompleteHandler<T, Env>;
}

export declare interface OptionProps {
    value: string;
    description?: string;
    emoji?: string | APIMessageComponentEmoji;
    default?: boolean;
    children?: Child[];
}

export declare interface Options<Env = unknown> {
    applicationId?: Snowflake;
    applicationPublicKey?: string;
    applicationSecret?: string;
    testServerId?: Snowflake;
    commands?: ChatInputCommands<Env>;
    userCommands?: UserCommands<Env>;
    messageCommands?: MessageCommands<Env>;
}

export declare interface RequiredChoicesOption<Choices extends ReadonlyArray<Choice<string | number>>> {
    required: true;
    choices: Choices;
}

export declare interface RequiredOption<T, Env> {
    required: true;
    autocomplete?: AutocompleteHandler<T, Env>;
}

export declare function Row(props: RowProps): APIActionRowComponent & {
    [$actionRow]: true;
};

export declare interface RowProps {
    children?: Exclude<APIMessageComponent, APIActionRowComponent>[];
}

export declare function Select(props: SelectProps): APISelectMenuComponent & {
    [$actionRowChild]: true;
};

export declare interface SelectProps {
    id: string;
    placeholder?: string;
    min?: number;
    max?: number;
    disabled?: boolean;
    children?: APISelectMenuOption[];
}

export declare function useBoolean<Env>(name: string, description: string, options?: OptionalOption<boolean, Env>): boolean | null;

export declare function useBoolean<Env>(name: string, description: string, options: RequiredOption<boolean, Env>): boolean;

export declare function useButton<Env>(handler: ComponentHandler<Env, APIMessageButtonInteractionData>): string;

export declare function useChannel<Env>(name: string, description: string, options?: OptionalOption<string, Env> & ChannelOption): APIInteractionDataResolvedChannel | null;

export declare function useChannel<Env>(name: string, description: string, options: RequiredOption<string, Env> & ChannelOption): APIInteractionDataResolvedChannel;

export declare function useDefaultPermission(permission: boolean): void;

export declare function useDescription(description: string): void;

export declare function useInteger<Env>(name: string, description: string, options?: OptionalOption<number, Env> & NumericOption): number | null;

export declare function useInteger<Env>(name: string, description: string, options: RequiredOption<number, Env> & NumericOption): number;

export declare function useInteger<Choices extends ReadonlyArray<Choice<number>>>(name: string, description: string, options: OptionalChoicesOption<Choices>): ChoiceValue<Choices[number]> | null;

export declare function useInteger<Choices extends ReadonlyArray<Choice<number>>>(name: string, description: string, options: RequiredChoicesOption<Choices>): ChoiceValue<Choices[number]>;

export declare function useMentionable<Env>(name: string, description: string, options?: OptionalOption<string, Env>): APIUser | APIRole | null;

export declare function useMentionable<Env>(name: string, description: string, options: RequiredOption<string, Env>): APIUser | APIRole;

export declare function useNumber<Env>(name: string, description: string, options?: OptionalOption<number, Env> & NumericOption): number | null;

export declare function useNumber<Env>(name: string, description: string, options: RequiredOption<number, Env> & NumericOption): number;

export declare function useNumber<Choices extends ReadonlyArray<Choice<number>>>(name: string, description: string, options: OptionalChoicesOption<Choices>): ChoiceValue<Choices[number]> | null;

export declare function useNumber<Choices extends ReadonlyArray<Choice<number>>>(name: string, description: string, options: RequiredChoicesOption<Choices>): ChoiceValue<Choices[number]>;

export declare type UserCommand<Env = unknown> = () => UserCommandHandler<Env>;

/** @see {CommandResponse} */
export declare type UserCommandHandler<Env = unknown> = (interaction: APIUserApplicationCommandInteraction, env: Env, ctx: ExecutionContext, user: APIUser) => CommandResponse;

export declare type UserCommands<Env = unknown> = Record<string, UserCommand<Env>>;

export declare function useRole<Env>(name: string, description: string, options?: OptionalOption<string, Env>): APIRole | null;

export declare function useRole<Env>(name: string, description: string, options: RequiredOption<string, Env>): APIRole;

export declare function useSelectMenu<Env>(handler: ComponentHandler<Env, APIMessageSelectMenuInteractionData>): string;

export declare function useString<Env>(name: string, description: string, options?: OptionalOption<string, Env>): string | null;

export declare function useString<Env>(name: string, description: string, options: RequiredOption<string, Env>): string;

export declare function useString<Choices extends ReadonlyArray<Choice<string>>>(name: string, description: string, options: OptionalChoicesOption<Choices>): ChoiceValue<Choices[number]> | null;

export declare function useString<Choices extends ReadonlyArray<Choice<string>>>(name: string, description: string, options: RequiredChoicesOption<Choices>): ChoiceValue<Choices[number]>;

export declare function useUser<Env>(name: string, description: string, options?: OptionalOption<string, Env>): APIUser | null;

export declare function useUser<Env>(name: string, description: string, options: RequiredOption<string, Env>): APIUser;

export declare type ValueOf<T> = T[keyof T];

export declare type WithFileAttachments<T extends {
    attachments?: APIInteractionResponseCallbackData["attachments"] | null;
}> = Omit<T, "attachments"> & {
    attachments?: File[];
};

export { }
